@@ -1,5 +1,5 @@
 package Process::Child::Leash;
-$Process::Child::Leash::VERSION = '000.002';
+$Process::Child::Leash::VERSION = '000.003';
 =head1 NAME
 
 Process::Child::Leash - to make sure the child process wont get lost with their parent.
@@ -30,32 +30,49 @@ the real process ( child ).
 
  ... start of the script ...
 
+ ## run.sh
+ #!/bin/bash
+ export SOMETHING=FOOBAR
+ perl script.pl
+
+ >> bash run.sh
+
+
 =head1 USE IT OUTSIDE THE CODE
 
+ ## run.sh
  #!/bin/bash
  export SOMETHING=FOOBAR
  perl -MProcess::Child::Leash script.pl
 
+ >> bash run.sh
+
 =head1 TIMEOUT THE HANGING PROCESS
 
 Timeout after 10 seconds running.
 
+ ## run.sh
  #!/bin/bash
  export CHILD_LEASH_TIMEOUT=10
  export DBIC_TRACE=1
  perl -MProcess::Child::Leash script.pl
-
- OR
-
  perl -MProcess::Child::Leash=timeout,10 script.pl
 
+ >> bash run.sh
+
+=head2 THEN WHAT?
+
+get the pid of run.sh, and kill -9 that pid.
+
+The script.pl process will be terminated.
+
 =cut
 
 use strict;
 use warnings;
-use Unix::PID;
-use Async;
 use Mouse;
+use Proc::Killfam qw( killfam );
+use Unix::PID;
 
 has _started_time => (
     is      => "ro",
@@ -66,12 +83,12 @@ has _started_time => (
 sub _build__started_time { time }
 
 has _child_pid => (
-    is      => "ro",
-    isa     => "Int",
-    builder => "_build__child_process",
+    is         => "rw",
+    isa        => "Int",
+    lazy_build => 1,
 );
 
-sub _build__child_process { $$ }
+sub _build__child_pid { $$ }
 
 has _parent_pid => (
     is      => "ro",
@@ -79,7 +96,12 @@ has _parent_pid => (
     builder => "_build__parent_pid",
 );
 
-sub _build__parent_pid { getppid() }
+sub _build__parent_pid {
+    my $parent_pid = getppid();
+    warn "PARENT ID: $parent_pid\n"
+      if $ENV{DEBUG_PROCESS_CHILD_LEASH};
+    return $parent_pid;
+}
 
 has timeout => (
     is      => "ro",
@@ -94,45 +116,64 @@ sub _build_timeout {
       || 0;
 }
 
+has test => (
+    is         => "ro",
+    isa        => "Bool",
+    lazy_build => 1,
+);
+
 no Mouse;
 
-sub import {
-    my $class = shift;
-    my %args  = @_;
-    return
-      if $args{test};
-    my $leash = $class->new(%args)->tieup;
-}
+{
+    my $IS_TEST = 0;
 
-sub tieup {
-    my $self = shift;
-    my $hook = Async->new( sub { $self->_hook } );
-    while ( !$hook->ready ) {
-        sleep 1;
+    sub _build_test {
+        return $IS_TEST;
+    }
+
+    sub import {
+        my $class = shift;
+        my %args  = @_;
+        return
+          if $IS_TEST = $args{test};
+        my $leash = $class->new(%args);
+        $leash->fasten;
     }
-    exit;
 }
 
-sub _hook {
+sub fasten {
+    my $self             = shift;
+    my $real_process_pid = fork;
+    if ($real_process_pid) {
+        $self->_child_pid($real_process_pid);
+        $self->_baby_sit;
+        exit;
+    }
+}
+
+sub _baby_sit {
     my $self       = shift;
     my $parent_pid = $self->_parent_pid;
     my $child_pid  = $self->_child_pid;
     my $start_at   = $self->_started_time;
     my $timeout    = $self->timeout;
 
+    warn ">> BABY SITTER: $$\n"
+      if $ENV{DEBUG_PROCESS_CHILD_LEASH};
+
   CHECK: while (1) {
         if ( $self->_is_timeout && $self->_kill_child_process ) {
-            return "timeout. killed child process";
+            return $self->finish("timeout. killed child process");
         }
         elsif ( !$self->_is_child_still_running ) {
-            return "child is gone. finish checking";
+            return $self->finish("child is gone. finish checking");
         }
         elsif ( $self->_is_parent_still_running ) {
             sleep 1;
             next CHECK;
         }
         elsif ( $self->_kill_child_process ) {
-            return "parent is gone. killed child process";
+            return $self->finish("parent is gone. killed child process");
         }
     }
 }
@@ -157,7 +198,11 @@ sub _hook {
     }
 }
 
-sub _kill_process { kill -9, shift }
+sub _kill_process {
+    my $self = shift;
+    my $pid  = shift;
+    killfam "TERM", $pid;
+}
 
 sub _is_timeout {
     my $self    = shift;
@@ -171,17 +216,31 @@ sub _is_timeout {
 sub _is_process_still_running {
     my $self = shift;
     my $pid  = shift;
-    return Unix::PID->new->pid_info($pid) ? 1 : 0;
+    my @info = Unix::PID->new->pid_info($pid);
+    return @info ? 1 : 0;
 }
 
 sub _is_parent_still_running {
     my $self = shift;
-    $self->_is_process_still_running($self->_parent_pid);
+    my $bool = $self->_is_process_still_running( $self->_parent_pid );
+    my @info = Unix::PID->new->pid_info( $self->_parent_pid );
+    warn "$bool - $#info - @info\n"
+      if $ENV{DEBUG_PROCESS_CHILD_LEASH};
+    return $bool;
+
 }
 
 sub _is_child_still_running {
     my $self = shift;
-    $self->_is_process_still_running($self->_child_pid);
+    $self->_is_process_still_running( $self->_child_pid );
+}
+
+sub finish {
+    my $self = shift;
+    exit
+      if !$self->test;
+    my $message = shift;
+    return $message;
 }
 
 1;
